@@ -22,13 +22,16 @@
 #
 
 import re
+
 import time
 from xml.sax.saxutils import escape as escape_mkup
 
-from gi.repository import Gtk as gtk
+from gi.repository import Gtk
 from gi.repository import GdkPixbuf, GObject, Pango
 
-from gettext import gettext as _
+import gettext
+#from gettext import gettext as _
+
 from .SimpleGladeApp import SimpleGladeApp
 
 from .config import *
@@ -62,8 +65,8 @@ class EditPartition(SimpleGladeApp) :
         self.dialog_edit.connect("delete-event", self.on_quit)
         self.cancel_button2.connect("clicked", self.on_quit)
         
-        # gtk.Tooltips().set_tip(self.default_button, _("Return options to defaults"))
-        # gtk.Tooltips().set_tip(self.browser_button, _("Select a directory"))
+        # Gtk.Tooltips().set_tip(self.default_button, _("Return options to defaults"))
+        # Gtk.Tooltips().set_tip(self.browser_button, _("Select a directory"))
          
         self.path.set_text(self.entry["FSTAB_PATH"])
         self.options3.set_text(self.entry["FSTAB_OPTION"])
@@ -78,13 +81,15 @@ class EditPartition(SimpleGladeApp) :
         if not driver in [ k[0] for k in drivers_list ] :
             if driver in self.drivers["all"] :
                 drivers_list.append(self.drivers["all"][driver])
+            elif driver == "ntfs3" :
+                drivers_list.append([driver, "Read-write kernel driver", 0])
             else :
                 drivers_list.append([driver, "Unknow driver", 0])
         if len(drivers_list) > 1 or not self.drivers["primary"] :
             self.vbox_driver.show()
             self.driver_box.clear()
-            self.liststore = gtk.ListStore(str)
-            renderer = gtk.CellRendererText()
+            self.liststore = Gtk.ListStore(str)
+            renderer = Gtk.CellRendererText()
             self.driver_box.set_model(self.liststore)
             self.driver_box.pack_start(renderer, True)
             self.driver_box.add_attribute(renderer, 'text', 0)
@@ -108,7 +113,8 @@ class EditPartition(SimpleGladeApp) :
         self.options3.set_text(self.entry.defaultopt())
 
     def set_driver(self) :
-        if self.entry["FS_TYPE"] in ('ntfs', 'exfat') :
+        fsk = 0
+        if self.entry["FS_TYPE"] in ('ntfs', 'ntfs3', 'exfat') :
             self.driver_warning_box.hide()
             self.driver_error_box.hide()
         if self.drivers["primary"] :
@@ -116,13 +122,14 @@ class EditPartition(SimpleGladeApp) :
                 self.driver_warning_box.hide()
                 fsk = self.drivers["all"][self.entry["FSTAB_TYPE"]][2]
             else :
-                self.driver_warning_box.show()
-                fsk = 0
+                if self.entry["FS_TYPE"] not in ('ntfs', 'ntfs3', 'exfat') :
+                    self.driver_warning_box.show()
+                    fsk = 0
         else :
             self.driver_box.set_sensitive(False)
             self.driver_error_box.show()
             self.driver_error_label.set_label( \
-               _("No driver is available for this type of filsystem : '%s'") \
+               _("No driver is available for this type of filesystem : '%s'") \
                % self.entry["FS_TYPE"])
             fsk = 0
         self.check_button.set_active(bool(int(self.entry["FSTAB_PASO"])))
@@ -130,33 +137,46 @@ class EditPartition(SimpleGladeApp) :
 
     def on_browser_clicked(self, button) :
     
-        #gladexml =  gtk.glade.XML(GLADEFILE, "dialog_mount_point")
-        #dialog = gladexml.get_widget("dialog_mount_point")
+        """  not used:
+        gladexml =  Gtk.glade.XML(GLADEFILE, "dialog_mount_point")
+        dialog = gladexml.get_widget("dialog_mount_point")
 
-        # builder = Gtk.Builder()
-        # builder.add_from_file(GLADEFILE)
+        builder = Gtk.Builder()
+        builder.add_from_file(GLADEFILE)
 
-        # dialog = builder.get_object("dialog_mount_point")
+        dialog = builder.get_object("dialog_mount_point")
 
-        # dialog.set_transient_for(self.dialog_edit)
-        # dialog.select_filename(self.path.get_text())
-        # for shortcut in (get_user("dir"), "/media", "/") :
-            # try :
-                # dialog.add_shortcut_folder(shortcut)
-            # except gobject.GError : 
-                # pass
-        # ret = dialog.run()
-        # print(ret)
-        # print(dialog.get_filename())
+        dialog.set_transient_for(self.dialog_edit)
+        dialog.select_filename(self.path.get_text())
+        for shortcut in (get_user("dir"), "/media", "/") :
+            try :
+                dialog.add_shortcut_folder(shortcut)
+            except gobject.GError : 
+                pass
+        ret = dialog.run()
+        print(ret)
+        print(dialog.get_filename())
         
-        # if ret == 1 :
-            # self.path.set_text(dialog.get_filename())
-        # dialog.hide()
+        if ret == 1 :
+            self.path.set_text(dialog.get_filename())
+        dialog.hide()
 
         dialog = Gtk.FileChooserDialog(title="Please choose a folder", action=Gtk.FileChooserAction.SELECT_FOLDER)
+        """
+        
+        dialog = Gtk.FileChooserDialog(title=_("Select a directory"), action=Gtk.FileChooserAction.SELECT_FOLDER)
+
+        gtk_select = "_Select"
+        
         dialog.add_buttons(
-            Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, "Select", Gtk.ResponseType.OK
+            Gtk.STOCK_CANCEL, 
+            Gtk.ResponseType.CANCEL, 
+            gettext.translation('gtk30').gettext(gtk_select), 
+            Gtk.ResponseType.OK
         )
+       #dialog.add_buttons(
+       #     Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, "Select", Gtk.ResponseType.OK
+       # )
 
         logging.debug("Mountpoint: " + self.path.get_text())
         user_mp_dir = os.path.dirname(os.path.realpath(self.path.get_text()))
@@ -185,9 +205,12 @@ class EditPartition(SimpleGladeApp) :
         current = self.driver_box.get_active_text().split()[0]
         if self.entry["DEVICE"] and not self.entry["FSTAB_TYPE"] == current :
             self.disk.set(self.entry, type=current)
+            # deduplicate
+            self.entry["FSTAB_OPTION"] = ','.join(dict.fromkeys(self.entry["FSTAB_OPTION"].split(',')))
+
             self.options3.set_text(self.entry["FSTAB_OPTION"])
             self.set_driver()
-        
+
     def on_apply(self, widget) :
     
         if not self.check_new_options() and not self.check_new_path() :
@@ -227,11 +250,13 @@ class EditPartition(SimpleGladeApp) :
             return 0     
         dialog("warning", _("Options formatting error"), \
                 _("<i>%s</i> is not formatted correctly.\n" \
-                "Options should be separated by coma (,)") % escape_mkup(options), \
+                "Options should be separated by comma (,)") % escape_mkup(options), \
                 parent = self.dialog_edit)
         self.dialog_edit.set_focus(self.options3)
         return 1
 
+""" 
+# (not used)
 
 class HistoryDialog(SimpleGladeApp) :
     ''' Class the manage the History dialog '''
@@ -251,11 +276,11 @@ class HistoryDialog(SimpleGladeApp) :
         self.close_button.connect("clicked", self.on_quit)
         self.dialog_history.connect("delete-event", self.on_quit)
         
-        renderer = gtk.CellRendererText()
-        column = gtk.TreeViewColumn("historic", renderer, text=0, editable=1)
+        renderer = Gtk.CellRendererText()
+        column = Gtk.TreeViewColumn("historic", renderer, text=0, editable=1)
         self.treeview.append_column(column)
         
-        self.tree_store = gtk.ListStore(str, bool)
+        self.tree_store = Gtk.ListStore(str, bool)
         self.treeview.set_model(self.tree_store)
         self.update_dialog()
         
@@ -281,7 +306,7 @@ class HistoryDialog(SimpleGladeApp) :
         self.apply_button.set_sensitive(False)
         version = self.versions[self.treeview.get_cursor()[0][0]]
         changes = self.disk.get_changes_current_to(version)
-        buf = gtk.TextBuffer()
+        buf = Gtk.TextBuffer()
         tag = buf.create_tag(weight=Pango.Weight.BOLD)
         buf.insert_with_tags(buf.get_start_iter(), "Commit:\n", tag)
         buf.insert_at_cursor(self.disk.get_logcommit(version))
@@ -298,19 +323,19 @@ class HistoryDialog(SimpleGladeApp) :
         ret = dialog("question", _("Reverting to an older version?"), \
             [_("This will apply the following changes:"), _("Do you want to continue?")],
             "\n".join(self.disk.get_changes_current_to(version)), parent = self.dialog_history)
-        if ret[0] == gtk.ResponseType.YES :
+        if ret[0] == Gtk.ResponseType.YES :
             self.disk.revert_to(version)
         self.on_cursor_changed(self.treeview)
         
     def on_see_clicked(self, button) :
     
         version = self.versions[self.treeview.get_cursor()[0][0]]
-        gladexml =  gtk.glade.XML(GLADEFILE, "dialog_fstab")
+        gladexml =  Gtk.glade.XML(GLADEFILE, "dialog_fstab")
         dialog = gladexml.get_widget("dialog_fstab")
         dialog.set_title("")
         dialog.set_transient_for(self.dialog_history)
         textview = gladexml.get_widget("textview_fstab")
-        buf = gtk.TextBuffer()
+        buf = Gtk.TextBuffer()
         if not version == "Original" :
             buf.set_text(FstabData.header)
         buf.insert_at_cursor(self.disk.get_logfile(version))
@@ -322,13 +347,14 @@ class HistoryDialog(SimpleGladeApp) :
     
         ret = dialog("question", _("Emptying History?"), \
             _("Do you want to empty the History?"), parent = self.dialog_history)
-        if ret[0] == gtk.ResponseType.YES :
+        if ret[0] == Gtk.ResponseType.YES :
             self.disk.cleanlog()
             self.update_dialog()
             
     def on_quit(self, button, event = None) :
     
         self.dialog_history.hide()
+"""
         
         
 class InfoDialog(SimpleGladeApp) :
@@ -343,8 +369,8 @@ class InfoDialog(SimpleGladeApp) :
         self.infobuttonclose.connect("clicked", self.on_quit)
 
     def update_dial(self) :
-
-        self.dialog_info.set_title(self.entry["DEV"] + " information")
+        gtk_information = "Information"
+        self.dialog_info.set_title(self.entry["DEV"] + " " + gettext.translation('gtk30').gettext(gtk_information))
         self.size_label.set_label(size_renderer(self.entry.get_size()))
         if self.entry.get_is_mounted() :
             self.free_label.set_label(size_renderer(self.entry.get_free_size()))
@@ -359,13 +385,26 @@ class InfoDialog(SimpleGladeApp) :
         self.dev_label.set_label(self.entry["DEVICE"])
         self.label_label.set_label(self.disk.get_attribute(self.entry, "FS_LABEL"))
         self.uuid_label.set_label(self.disk.get_attribute(self.entry, "FS_UUID"))
-        self.type_label.set_label("%s (%s)" % \
-                (self.disk.get_attribute(self.entry, "FS_TYPE"), \
-                self.disk.get_attribute(self.entry, "FS_VERSION")))
-        self.vendor_label.set_label(self.disk.get_attribute(self.entry, "VENDOR"))
-        self.model_label.set_label(self.disk.get_attribute(self.entry, "MODEL"))
-        self.serial_label.set_label(self.disk.get_attribute(self.entry, "SERIAL"))
-        self.bus_label.set_label(self.disk.get_attribute(self.entry, "BUS"))
+
+        if self.disk.get_attribute(self.entry, "FS_VERSION") in "None":
+            self.type_label.set_label("%s" % \
+                    (self.disk.get_attribute(self.entry, "FS_TYPE")))
+        else:
+            self.type_label.set_label("%s (%s)" % \
+                    (self.disk.get_attribute(self.entry, "FS_TYPE"), \
+                    self.disk.get_attribute(self.entry, "FS_VERSION")))
+
+        if self.disk.get_attribute(self.entry, "VENDOR") not in "None":
+            self.vendor_label.set_label(self.disk.get_attribute(self.entry, "VENDOR"))
+
+        if self.disk.get_attribute(self.entry, "MODEL") not in "None":
+            self.model_label.set_label(self.disk.get_attribute(self.entry, "MODEL"))
+
+        if self.disk.get_attribute(self.entry, "SERIAL") not in "None":
+            self.serial_label.set_label(self.disk.get_attribute(self.entry, "SERIAL"))
+
+        if self.disk.get_attribute(self.entry, "BUS") not in "None":
+            self.bus_label.set_label(self.disk.get_attribute(self.entry, "BUS"))
         
     def on_quit(self, button, event = None) :
     
@@ -400,7 +439,7 @@ class SanityCheck :
                 "at start-up. Select the ones you want to remove.")], \
                 [ "%s on %s" % (k["FSTAB_NAME"], k["FSTAB_PATH"]) for k in unknow_block ], \
                 _("Remove selected"), [_("Don't show me this warning in the future.")], self.parent)
-            if not dial[0] ==  gtk.ResponseType.REJECT :
+            if not dial[0] ==  Gtk.ResponseType.REJECT :
                 for i in dial[1][0] :
                     logging.debug("Removing unknow entry : %s" % unknow_block[i]["FSTAB_NAME"])
                     self.fstab.other.remove(unknow_block[i])
@@ -440,7 +479,7 @@ class SanityCheck :
                 "You should select the one you want, and the others will be commented.\n"
                 "They will be kept in the configuration file, but will be disabled.")], \
                 data, _("Keep selected"), parent = self.parent)
-                if not dial[0] ==  gtk.ResponseType.REJECT :
+                if not dial[0] ==  Gtk.ResponseType.REJECT :
                     for j in dial[1][1] :
                         i = index[j]
                         logging.debug("Commenting duplicate : %s" % entries[j])
@@ -454,8 +493,8 @@ class SanityCheck :
         
 def about_dialog(button, parent) :
 
-    dial = gtk.AboutDialog()
-    dial.set_name("Disk Manager")
+    dial = Gtk.AboutDialog()
+    dial.set_name(_("Disk Manager"))
     dial.set_version(VERSION)
     dial.set_copyright("Copyright © 2007 Mertens Florent \n 2021 MX Linux")
     dial.set_license( \
@@ -472,7 +511,7 @@ def about_dialog(button, parent) :
         "You should have received a copy of the GNU General Public License\n"
         "along with this library; if not, write to the Free Software\n"
         "Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA")
-    dial.set_comments("Easily manage your filesystem configuration")
+    dial.set_comments(_("Easily manage your filesystem configuration"))
     dial.set_website(HOMEPAGE)
     dial.set_website_label("https://github.com/MX-Linux/disk-manager")
     dial.set_authors(AUTHORS)
